#!/usr/bin/env python

# GUI Rnmap client v. 0.10
#
# Copyright (C) 2000-2003 Tuomo Makinen (tmakinen@pp.htv.fi)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
Rnmap GUI client v. 0.10
"""

import sys
import string
import os
import time
try:
    from Tkinter import *
    from tkFileDialog import *
except ImportError:
    sys.stderr.write("Can't find Tkinter libraries, exiting ...\n")
    sys.exit(1)
try:
    import threading
except ImportError:
    sys.stderr.write("grnmap needs python compiled with threads.\n")
    sys.exit(1)

sys.path[:0] = ['../lib']

import clientsocket
from client import *

class Connection_handler:
    def __init__(self):
        self.scanintr = 0
        self.use_ssl_socket = gui.ssl_socket.get()
        
    def do_connection(self, message):
        # Use SSL or normal socket
        if self.use_ssl_socket:
            self.my_connection = clientsocket.SSLClient(gui.verify_cb, CA_CERTS, 1)
        else:
            self.my_connection = clientsocket.RegularClient()
        # Connect to server
        con_stat = self.my_connection.open_connection(Server, Port)
        if con_stat[0] == -1:
            gui.infomsg.set(con_stat[1])
            return -1
        # Send message to server
        if self.my_connection.write_socket(message) == -1:
            gui.infomsg.set(SENDERR)
            return -1
        # Receive data from socket
        workbuf = ''; cnt = 0
        while 1:
            if self.use_ssl_socket and self.scanintr:
                time.sleep(0.5)
            chunk = self.my_connection.read_socket()
            if not chunk:
                break
            elif chunk == -1:
                gui.infomsg.set(RECVERR)
                break
            workbuf = workbuf + chunk
            # Scan delimiter (CRLF) and handle message
            while 1:
                index = string.find(workbuf, CRLF)
                if index == -1:
                    break  
                scan_running = gui.handle_message(workbuf[:index], cnt)
                if not cnt and scan_running == -1: # Msg type is not 201
                    return -1
                workbuf = workbuf[index+2:]
                cnt = cnt + 1
        if not cnt:
            return -1
        
    def shutdown_connection(self):
        self.scanintr = 1
        gui.infomsg.set("Interrupting scan...")
        self.my_connection.write_socket(INTERRUPT_SCAN)

    def close_connection(self):
        gui.running_scan = 0
        self.my_connection.close_socket()
            
class Rgui:
    def __init__(self, master):
        scanopt_pad = 2
        self.running_scan = 0
	self.font_1 = None
	self.logparam = IntVar()
    	self.append_log = IntVar()
    	self.ssl_socket = IntVar()
	self.scanslct	= IntVar()
	self.pingslct   = IntVar()
	self.sIvar 	= IntVar()
	self.sUvar 	= IntVar()
	self.nRvar 	= IntVar()
	self.oSvar 	= IntVar()
	self.vBvar 	= IntVar()
	self.sRvar	= IntVar()
	self.fragIP	= IntVar()
	self.fastscan	= IntVar()
	self.srcbutton	= IntVar()
	self.portbutton	= IntVar()
	self.guruknow 	= IntVar()
	self.decoybtn	= IntVar()
	self.slctbounce	= IntVar()
	self.keep_pass  = IntVar()
	self.accept_value = IntVar()
	self.infomsg 	= StringVar()
	self.servername = StringVar()
	self.null  	= StringVar()
	self.ports 	= StringVar()
	self.hostlist	= StringVar()
	self.sourceport = StringVar()
	self.servername.set(Server + ":" + `Port`)
	self.scanslct.set(2)
	self.pingslct.set(2)
	self.logparam.set(1)
	self.infomsg.set("Welcome to Remote nmap")
        # --
        self.mbar = Menu(master, relief=GROOVE, borderwidth=2)
        master.config(menu=self.mbar)
        self.filecommand()
    	self.logcommand()
    	self.sslcommand()
	self.helpcommand()
        # --
        self.main_frame    = Frame(master, relief=GROOVE, borderwidth=0)
        self.main_frame.grid(row=0, column=0, rowspan=5, columnspan=3, sticky=W)
        self.scanopt_frame        = Frame(master, relief=GROOVE, borderwidth=2)
        self.scanopt_frame.grid(row=1, column=2, rowspan=14, padx=7, pady=11, ipady=10)
        self.gnrlopt_frame        = Frame(master, relief=GROOVE, borderwidth=2)
        self.gnrlopt_frame.grid(row=6, column=0, rowspan=9, columnspan=2,sticky=W, padx=3, pady=11, ipady=10)
	for text, row, column, fg, sticky, padx, pady, rowspan in [("Host(s) to scan:", 1, 0, None, W, 3, 0, 1), ("Login ID:", 2, 0, None, W, 3, 0, 1), ("Password:", 3, 0, None, W, 3, 0, 1), \
                                              ("Server:", 4, 0, None, W, 3, 0, 1)]:
                Label(self.main_frame, text=text, font=self.font_1, fg=fg).grid(row=row, column=column, sticky=sticky, padx=padx, pady=pady, rowspan=rowspan)

        self.scanopt = Label(master, text="Scan options:", font=self.font_1, fg="darkblue").grid(row=1, column=2, sticky=NW, padx=scanopt_pad + 10, pady=1, rowspan=2)
        self.gnrlopt = Label(master, text="General scan options:", font=self.font_1, fg="darkblue").grid(row=6, column=0, sticky=NW, padx=8, pady=1, rowspan=1)

        self.scan_results = Text(master, height=18, width=0, bg="white")
        self.scan_results.grid(row=15, column=0, columnspan=4, sticky=NSEW)
        # --
	self.logid      = Entry(self.main_frame, width = 20, bg="white")
	self.secret     = Entry(self.main_frame, width = 20, bg="white", show="*", textvariable=self.null)
	self.hosts	= Entry(self.main_frame, width = 28, bg="white", textvariable=self.hostlist)
    	self.server	= Entry(self.main_frame, width = 20, bg="white", textvariable=self.servername)
	self.sC 	= Radiobutton(self.scanopt_frame, text="connect()    ", font=self.font_1, variable=self.scanslct, value=1, command=self.check_scanoptions)
    	self.sS		= Radiobutton(self.scanopt_frame, text="SYN scan     ", font=self.font_1, variable=self.scanslct, value=2, command=self.check_scanoptions)
    	self.sN		= Radiobutton(self.scanopt_frame, text="NULL scan    ", font=self.font_1, variable=self.scanslct, value=3, command=self.check_scanoptions)
    	self.sF		= Radiobutton(self.scanopt_frame, text="FIN scan     ", font=self.font_1, variable=self.scanslct, value=4, command=self.check_scanoptions)
    	self.sX		= Radiobutton(self.scanopt_frame, text="XMAS scan    ", font=self.font_1, variable=self.scanslct, value=5, command=self.check_scanoptions)
    	self.sA		= Radiobutton(self.scanopt_frame, text="ACK scan     ", font=self.font_1, variable=self.scanslct, value=6, command=self.check_scanoptions)
    	self.sW		= Radiobutton(self.scanopt_frame, text="Window scan  ", font=self.font_1, variable=self.scanslct, value=7, command=self.check_scanoptions)
	self.sP 	= Radiobutton(self.scanopt_frame, text="Ping scan    ", font=self.font_1, variable=self.scanslct, value=8, command=self.check_scanoptions)
        self.sL         = Radiobutton(self.scanopt_frame, text="List scan    ", font=self.font_1, variable=self.scanslct, value=9, command=self.check_scanoptions)
        self.sPR        = Radiobutton(self.scanopt_frame, text="Protocol scan", font=self.font_1, variable=self.scanslct, value=10, command=self.check_scanoptions)
	self.bouncebtn	= Checkbutton(self.scanopt_frame, text="Bounce scan: ", font=self.font_1, variable=self.slctbounce, command=self.check_bouncestatus)
	self.bouncebox	= Entry(self.scanopt_frame, width = 13, bg="white")
    	self.oS 	= Checkbutton(self.gnrlopt_frame, text="OS Detect    ", font=self.font_1, variable=self.oSvar)
	self.nR 	= Checkbutton(self.gnrlopt_frame, text="Don't resolve ", font=self.font_1, variable=self.nRvar)
	self.vB 	= Checkbutton(self.gnrlopt_frame, text="Verbose      ", font=self.font_1, variable=self.vBvar)
	self.sI 	= Checkbutton(self.gnrlopt_frame, text="Ident scan   ", font=self.font_1, variable=self.sIvar)
	self.sU 	= Checkbutton(self.gnrlopt_frame, text="UDP scan     ", font=self.font_1, variable=self.sUvar)
	self.guru	= Checkbutton(self.gnrlopt_frame, text="User options:", font=self.font_1, variable=self.guruknow)
	self.guruwrite	= Entry(self.gnrlopt_frame, width = 13, bg="white")
	self.sR		= Checkbutton(self.gnrlopt_frame, text="RPC scan     ", font=self.font_1, variable=self.sRvar)
	self.frag	= Checkbutton(self.gnrlopt_frame, text="Fragment IP  ", font=self.font_1, variable=self.fragIP)
	self.fast	= Checkbutton(self.gnrlopt_frame, text="Fast scan    ", font=self.font_1, variable=self.fastscan, command=lambda : self.portbutton.set(0))
	self.ports	= Checkbutton(self.gnrlopt_frame, text="Port range   ", font=self.font_1, variable=self.portbutton, command=lambda : self.fastscan.set(0))
	self.prange	= Entry(self.gnrlopt_frame, width = 13, bg="white")
	self.usedecoys	= Checkbutton(self.gnrlopt_frame, text="Use Decoys:  ", font=self.font_1, variable=self.decoybtn, command=self.check_decoystatus)
	self.decoys	= Entry(self.gnrlopt_frame, width = 13, bg="white")
	self.pT         = Radiobutton(self.gnrlopt_frame, text="TCP ping ", font=self.font_1, variable=self.pingslct, value=1)
	self.pIT        = Radiobutton(self.gnrlopt_frame, text="TCP&ICMP ", font=self.font_1, variable=self.pingslct, value=2)
	self.pI         = Radiobutton(self.gnrlopt_frame, text="ICMP ping ", font=self.font_1, variable=self.pingslct, value=3)
	self.pS         = Radiobutton(self.gnrlopt_frame, text="SYN ping ", font=self.font_1, variable=self.pingslct, value=4)
	self.pN         = Radiobutton(self.gnrlopt_frame, text="Don't ping ", font=self.font_1, variable=self.pingslct, value=5)
	self.srcbtn	= Checkbutton(self.gnrlopt_frame, text="Source port: ", font=self.font_1, variable=self.srcbutton)
    	self.src	= Entry(self.gnrlopt_frame, width = 13, bg="white", textvariable=self.sourceport)
	# --
        self.scan	= Button(self.main_frame, width=3, text="Scan", font=self.font_1, borderwidth=2, command=self.startwork)
        self.quit 	= Button(self.main_frame, width=3, text="Exit", font=self.font_1, borderwidth=2, command=self.quit_app)
	self.passrememb = Checkbutton(self.main_frame, text="Remember password", font=self.font_1, variable=self.keep_pass, command=self.disable_passwd)
        # --
	self.scroll 	= Scrollbar(master, command=self.scan_results.yview)
	self.message    = Message(master, font=self.font_1, fg = "blue", width=200, textvariable=self.infomsg)		
	self.scan_results.configure(yscrollcommand=self.scroll.set)
	self.logid.grid(row=2, column=1, padx=10, sticky=W)
	self.secret.grid(row=3, column=1, padx=10, sticky=W)
	self.hosts.grid(row=1, column=1, padx=10, sticky=W)
	self.server.grid(row=4, column=1, padx=10, sticky=W)
	self.sC.grid(row=0, sticky=W, padx=scanopt_pad)
	self.sS.grid(row=1, sticky=W, padx=scanopt_pad)
	self.sN.grid(row=2, sticky=W, padx=scanopt_pad)
	self.sF.grid(row=3, sticky=W, padx=scanopt_pad)
	self.sX.grid(row=4, sticky=W, padx=scanopt_pad)
	self.sA.grid(row=5, sticky=W, padx=scanopt_pad)
	self.sW.grid(row=6, sticky=W, padx=scanopt_pad)
	self.sP.grid(row=7, sticky=W, padx=scanopt_pad)
	self.sL.grid(row=8, sticky=W, padx=scanopt_pad)
	self.sPR.grid(row=9, sticky=W, padx=scanopt_pad)
	self.bouncebtn.grid(row=10, sticky=W, padx=scanopt_pad)
	self.bouncebox.grid(row=11, sticky=W, padx=scanopt_pad)
	self.oS.grid(row=0, sticky=W, column=0, padx=scanopt_pad)
    	self.nR.grid(row=1, sticky=W, column=0, padx=scanopt_pad)
	self.vB.grid(row=2, sticky=W, column=0, padx=scanopt_pad)
    	self.sI.grid(row=3, sticky=W, column=0, padx=scanopt_pad)
    	self.sU.grid(row=4, sticky=W, column=0, padx=scanopt_pad)
    	self.guru.grid(row=5, sticky=W, column=0, padx=scanopt_pad)
	self.guruwrite.grid(row=6, sticky=W, column=0, padx=scanopt_pad)
    	self.sR.grid(row=0, sticky=W, column=1)
	self.frag.grid(row=1, sticky=W, column=1)
    	self.fast.grid(row=2, sticky=W, column=1)
	self.ports.grid(row=3, sticky=W, column=1)
	self.prange.grid(row=4, sticky=W, column=1)
    	self.usedecoys.grid(row=5, sticky=W, column=1)
	self.decoys.grid(row=6, sticky=W, column=1)
	self.pT.grid(row=0, sticky=W, column=2)
	self.pIT.grid(row=1, sticky=W, column=2)
	self.pI.grid(row=2, sticky=W, column=2)
	self.pS.grid(row=3, sticky=W, column=2)
	self.pN.grid(row=4, sticky=W, column=2)
	self.srcbtn.grid(row=5, sticky=W, column=2)
	self.src.grid(row=6, sticky=W, column=2)
	self.scan.grid(row=5, column=0, sticky=NW, padx=3, rowspan=3)
  	self.quit.grid(row=5, column=0, sticky=NE, rowspan=3)
    	self.passrememb.grid(row=5, column=1, sticky=W, padx=10)
        self.scroll.grid(row=15, sticky=NS, column=4)
    	self.message.grid(row=16, columnspan=3, sticky=W)
            
    def configure_connect_scan(self):
        self.fragIP.set(0)
        self.decoybtn.set(0)
        self.srcbutton.set(0)
        self.frag.configure(state = DISABLED)
        self.usedecoys.configure(state = DISABLED)
        self.srcbtn.configure(state = DISABLED)
        self.src.configure(state = DISABLED)
        self.sI.configure(state = NORMAL)

    def configure_ping_scan(self):
        self.slctbounce.set(0)
        self.sUvar.set(0)
        self.oSvar.set(0)
        self.fastscan.set(0)
        self.portbutton.set(0)
        self.sRvar.set(0)
        if self.pingslct.get() == 5:
            self.pingslct.set(2)
        self.pN.configure(state = DISABLED)
        self.bouncebtn.configure(state = DISABLED)
        self.bouncebox.configure(state = DISABLED)
        self.sU.configure(state = DISABLED)
        self.oS.configure(state = DISABLED)
        self.fast.configure(state = DISABLED)
        self.ports.configure(state = DISABLED)
        self.prange.configure(state = DISABLED)
        self.sR.configure(state = DISABLED)
        
    def configure_general_scan(self):
        if self.scanslct.get() != 1:
            if self.scanslct.get() in (9, 10):
                self.fragIP.set(0)
                self.frag.configure(state = DISABLED)
                if self.scanslct.get() == 9:
                    self.fastscan.set(0)
                    self.portbutton.set(0)
                    self.fast.configure(state = DISABLED)
                    self.ports.configure(state = DISABLED)
                    self.prange.configure(state = DISABLED)
                else:
                    self.sUvar.set(0)
                    self.sRvar.set(0)
                    self.sU.configure(state = DISABLED)
                    self.sR.configure(state = DISABLED)
            else:
                self.frag.configure(state = NORMAL)
            self.sIvar.set(0)
            self.sI.configure(state = DISABLED)
            self.usedecoys.configure(state = NORMAL)
            self.srcbtn.configure(state = NORMAL)
            self.src.configure(state = NORMAL)
        if self.scanslct.get() != 8:
            self.bouncebtn.configure(state = NORMAL)
            self.bouncebox.configure(state = NORMAL)
            self.oS.configure(state = NORMAL)
            self.pN.configure(state = NORMAL)
            if self.scanslct.get() != 10:
                self.sU.configure(state = NORMAL)
                self.sR.configure(state = NORMAL)
            if self.scanslct.get() != 9:
                self.fast.configure(state = NORMAL)
                self.ports.configure(state = NORMAL)
                self.prange.configure(state = NORMAL)

    def check_scanoptions(self):
        if self.scanslct.get() == 1:
            self.configure_connect_scan()
            self.configure_general_scan()
        elif self.scanslct.get() == 8:
            self.configure_ping_scan()
            self.configure_general_scan()
        else:
            self.configure_general_scan()
                
    def filecommand(self):
	self.filemenu = Menu(self.mbar)
	self.filemenu.add_command(label="Save log", font=self.font_1, underline=0, command=self.savefile)
	self.filemenu.add_command(label="Open log", font=self.font_1, underline=0, command=self.openfile)
    	self.filemenu.add_command(label="Import hostlist", font=self.font_1, underline=0, command=self.loadlist)
	self.filemenu.add("separator")
    	self.filemenu.add_command(label="Exit", font=self.font_1, underline=1, command=self.quit_app)
    	self.mbar.add_cascade(label="File", menu=self.filemenu, font=self.font_1, underline=0)
	
    def logcommand(self):
    	self.logmenu = Menu(self.mbar)
    	self.logmenu.add_checkbutton(label="Append log", font=self.font_1, underline=0, variable=self.append_log)
    	self.logmenu.add("separator")
    	self.logmenu.add_radiobutton(label="Human readable", font=self.font_1, underline=0, variable=self.logparam, value=1)
	self.logmenu.add_radiobutton(label="Machine parseable", font=self.font_1, underline=0, variable=self.logparam, value=2)
	self.logmenu.add_radiobutton(label="XML output", font=self.font_1, underline=0, variable=self.logparam, value=4)
	self.logmenu.add_radiobutton(label="Script kiddie", font=self.font_1, underline=0, variable=self.logparam, value=3)
    	self.mbar.add_cascade(label="Log", menu=self.logmenu, font=self.font_1, underline=0)

    def sslcommand(self):
        self.sslmenu = Menu(self.mbar)
        if clientsocket.UseSSL == 1:
            self.ssl_socket.set(1)
            self.sslmenu.add_checkbutton(label="Use SSL", font=self.font_1, underline=0, variable=self.ssl_socket)
        else:
            self.ssl_socket.set(0)
            self.sslmenu.add_checkbutton(label="Use SSL", font=self.font_1, state = DISABLED, underline=0)
        self.mbar.add_cascade(label="SSL", menu=self.sslmenu, font=self.font_1, underline=0)
        
    def helpcommand(self):
        self.helpmenu = Menu(self.mbar)
	self.helpmenu.add_command(label="About", font=self.font_1, underline=0, command=self.aboutbox)
	self.mbar.add_cascade(label="Help", menu=self.helpmenu, font=self.font_1, underline=0)
        
    def aboutbox(self):
	self.about = Toplevel()
	Message(self.about, width=300, text="Rnmap gui client Version 0.10\nAuthor: Tuomo Makinen\nE-mail: tmakinen@pp.htv.fi", font=self.font_1, bg="white").grid()	
	Button(self.about, text="Close", font=self.font_1, command=lambda : self.about.withdraw()).grid(row=2, sticky=N)

    def check_decoystatus(self):
        if self.decoybtn.get() == 1:
            self.slctbounce.set(0)

    def check_bouncestatus(self):
        if self.slctbounce.get() == 1:
            self.decoybtn.set(0)

    def disable_passwd(self):
        if self.keep_pass.get() == 0:
            self.null.set("")

    def done(self):
        if self.infomsg.get() == "Scan running...":
            self.infomsg.set("Scan completed")
        elif self.infomsg.get() == "Interrupting scan...":
            self.infomsg.set("Scan interrupted")
        elif self.infomsg.get() == "Connecting to server...":
            self.infomsg.set("Server closed connection")
        self.scan.configure(command = self.startwork, state = NORMAL, relief = RAISED)
        self.change_cursor("")

    def change_cursor(self, type):
        root.configure(cursor=type)
        
    def savefile(self):
        filename = asksaveasfilename(filetypes=[("All Files", "*")])
        if filename:
            log = self.scan_results.get(1.0, END)
            fp = open(filename, "w")
            fp.write(log)
            fp.close()
            self.infomsg.set("Log saved")
		
    def openfile(self):
        filename = askopenfilename(filetypes=[("All Files", "*")])
        if filename:
            fp = open(filename, "r")
            res = fp.read()
            if self.append_log.get() == 0:
                self.scan_results.delete(1.0, END)
            else:
                self.scan_results.insert(END, "\n")
            self.scan_results.insert(END, res)
            self.infomsg.set("Log loaded")
    
    def loadlist(self):
        filename = askopenfilename(filetypes=[("All Files", "*")])
        if filename:
            fp = open(filename, "r")
            scanlist = fp.read()
            scanlist = string.replace(scanlist, "\012", "")
            self.hostlist.set(scanlist)   

    def startwork(self):
        self.scan.configure(state = DISABLED, relief = SUNKEN)
        self.infomsg.set("Connecting to server...")
        self.scan.update()
        Cv.acquire()
        threading.Thread(group=None, target=self.execute_scanrequest, args=()).start()
        Cv.wait()
        if self.running_scan == 1:
            self.scan.configure(command = self.stop_scan, state = NORMAL)
            self.infomsg.set("Scan running...")
            self.change_cursor("watch")
        Cv.release()
        
    def fetch_servervalue(self):
    	global Server, Port
    	sp = string.split(self.server.get(), ":")
    	Server = sp[0]
	try:
            Port = string.atoi(sp[1])
        except:
            self.infomsg.set("Invalid Server value")
            return -1
        # Generate new configuration file
        self.generate_configuration()

    def generate_configuration(self):
        try:
            if os.name == "posix":
                fp = open(os.environ["HOME"] + "/.grnmap.conf", "w")
            else:
                fp = open("grnmap.conf", "w")
            confstatus = "# Rnmap client configuration file\n\nport: " + `Port` + "\n" + "server: " + Server + "\n"
	    fp.write(confstatus)
	    fp.close()
        except IOError, e:
	    sys.stderr.write("Can't write to disk: " + e[1] + "\n")

    def fetch_nmapargs(self):
        nmap_cmd = ''
        # Scan options
    	if self.scanslct.get() == 1:
	    nmap_cmd = nmap_cmd + "-sT "
	elif self.scanslct.get() == 2:
	    nmap_cmd = nmap_cmd + "-sS "
	elif self.scanslct.get() == 3:
	    nmap_cmd = nmap_cmd + "-sN "
	elif self.scanslct.get() == 4:
	    nmap_cmd = nmap_cmd + "-sF "
	elif self.scanslct.get() == 5:
	    nmap_cmd = nmap_cmd + "-sX "
	elif self.scanslct.get() == 6:
	    nmap_cmd = nmap_cmd + "-sA "
	elif self.scanslct.get() == 7:
	    nmap_cmd = nmap_cmd + "-sW "
	elif self.scanslct.get() == 8:
	    nmap_cmd = nmap_cmd + "-sP "
	elif self.scanslct.get() == 9:
            nmap_cmd = nmap_cmd + "-sL "
        elif self.scanslct.get() == 10:
            nmap_cmd = nmap_cmd + "-sO "
        # Ping options
        if self.pingslct.get() == 1:
            nmap_cmd = nmap_cmd + "-PT "
        elif self.pingslct.get() == 3:
            nmap_cmd = nmap_cmd + "-PI "
        elif self.pingslct.get() == 4:
            nmap_cmd = nmap_cmd + "-PS "
        elif self.pingslct.get() == 5:
            nmap_cmd = nmap_cmd + "-P0 "
        # General options
	if self.sIvar.get() == 1:
	    nmap_cmd = nmap_cmd + "-I "
	if self.sUvar.get() == 1:
	    nmap_cmd = nmap_cmd + "-sU "
	if self.nRvar.get() == 1:
	    nmap_cmd = nmap_cmd + "-n "
	if self.oSvar.get() == 1:
	    nmap_cmd = nmap_cmd + "-O "
	if self.vBvar.get() == 1:
	    nmap_cmd = nmap_cmd + "-v "
	if self.sRvar.get() == 1:
	    nmap_cmd = nmap_cmd + "-sR "
	if self.fragIP.get() == 1:
	    nmap_cmd = nmap_cmd + "-f "
	if self.fastscan.get() == 1:
	    nmap_cmd = nmap_cmd + "-F "
	if self.logparam.get() == 1:
	    self.logformat = " -oN"
	elif self.logparam.get() == 2:
	    self.logformat = " -oM"
	elif self.logparam.get() == 3:
            self.logformat = " -oS"
        elif self.logparam.get() == 4:
            self.logformat = " -oX"
	if self.guruknow.get() == 1:
	    nmap_cmd = nmap_cmd + (self.guruwrite.get()+" ")
	if self.slctbounce.get() == 1:
	    nmap_cmd = nmap_cmd + ("-b "+self.bouncebox.get()+" ")
	if self.decoybtn.get() == 1:
	    nmap_cmd = nmap_cmd + ("-D "+self.decoys.get()+" ")
	if self.srcbutton.get() == 1:
	    nmap_cmd = nmap_cmd + ("-g "+self.sourceport.get()+" ")
	if self.scanslct.get() == 8 or self.portbutton.get() == 0:
	    pass
	else:
	    nmap_cmd = nmap_cmd + ("-p "+self.prange.get()+" ")
	nmap_cmd = nmap_cmd + (self.hosts.get())
	nmap_cmd = string.replace(nmap_cmd, ":", "###")
	return nmap_cmd
    
    def execute_scanrequest(self):
        if self.fetch_servervalue() == -1:
            notify_thread(0)
        else:
            user_passwd = self.secret.get()
            if self.keep_pass.get() == 0: self.null.set("")
            message = "101 " + self.logid.get() + ":" + user_passwd + ":" + self.fetch_nmapargs() + self.logformat
            self.connector = Connection_handler()
            if self.connector.do_connection(message) == -1:
                notify_thread(0)
            self.connector.close_connection()
        self.done()
        sys.exit()
    
    def handle_message(self, server_msg, round):
        if round == 0:
            if server_msg[:3] != "201":
                self.display_exception(server_msg[:3])
                return -1
            notify_thread(1)
            if self.append_log.get() == 0:
                self.scan_results.delete(1.0, END)
            else:
                if self.scan_results.index(END) != "2.0":
                    self.scan_results.insert(END, '\n')
            server_msg = server_msg[4:]
        # --
        self.scan_results.insert(END, server_msg)
        self.scan_results.see(self.scan_results.index(END))

    def display_exception(self, code):
        try:
            self.infomsg.set(PROTOCOL_CODES[code])
        except KeyError:
            self.infomsg.set(PROTOCOL_ERROR)

    def stop_scan(self):
        if self.running_scan == 1:
            try:
                self.connector.shutdown_connection() 
            except:
                pass
            
    def quit_app(self):
        os._exit(0)

    def verify_cb(self, conn, cert, errnum, depth, ok):
        if not ok:
            validate_cert = Check_certificate()
            valid = validate_cert.do_data(cert, errnum, depth)
            if errnum == 18 and valid: # Self signed certificate
                store_certificate(crypto.dump_certificate(crypto.FILETYPE_PEM, cert))
                ok = valid
        return ok

class Check_certificate:
    def __init__(self):
        self.cert_status = IntVar()
        
    def do_data(self, cert, errnum, depth):
        self.issuer = "Issuer = %s\n" % cert.get_issuer()
        self.subject = "Subject = %s\n" % cert.get_subject()
        self.md5digest = "MD5 Digest = %s\n" % digest_cert(crypto.dump_certificate(crypto.FILETYPE_ASN1, cert))
        self.cert_error = "** Can't verify certificate ** Error " + `errnum` + " at depth " + `depth` + "\n\n"
        if errnum == 18:
            self.prompt_user()
        else:
            self.reject_certificate()
        return self.cert_status.get()
    
    def reject_certificate(self):
        self.rejected_cert = Toplevel()
        self.rejected_cert.title("Can't verify certificate")
        Message(self.rejected_cert, text=self.cert_error + self.issuer + self.subject + self.md5digest + \
                "\n\nCertificate was rejected.\n", bg="white").grid()
        Button(self.rejected_cert, text="Ok", command=lambda : self.rejected_cert.withdraw()).grid(row=2, sticky=N)
        
    def prompt_user(self):
        self.verify_cert = Toplevel()
        self.verify_cert.bind('<Destroy>', self.abort)
        self.verify_cert.title("Self signed certificate")
	Message(self.verify_cert, text=self.cert_error + self.issuer + self.subject + self.md5digest + \
                "\n\nDo you accept certificate?\n", bg="white").grid()
	Button(self.verify_cert, text="Yes", command=self.accept_cert).grid(row=2, sticky=NW)
	Button(self.verify_cert, text="No", command=self.reject_cert).grid(row=2, sticky=NE)
	self.verify_cert.grab_set()
    	self.verify_cert.mainloop()
	try:
            self.verify_cert.grab_release()
            self.verify_cert.withdraw()
        except TclError:
            pass

    def produce_value(self, value):
        self.verify_cert.unbind('<Destroy>')
        self.cert_status.set(value)
        self.verify_cert.quit()

    def accept_cert(self):
        self.produce_value(1)

    def reject_cert(self):
        self.produce_value(0)

    def abort(self, event):
        self.produce_value(0)

def notify_thread(status):
    Cv.acquire()
    gui.running_scan = status
    Cv.notify()
    Cv.release()

def read_settings():
    try:
        if os.name == "posix":
            fp = open(os.environ["HOME"] + "/.grnmap.conf")
        else:
            fp = open("grnmap.conf")
        res = fp.readlines()
        fp.close()
        for ol in xrange(len(res)):
            for il in xrange(len(res[ol])):
                if res[ol][0] == "#":
                    continue
                elif res[ol][0] == " ":
                    continue
                elif res[ol][il] == ":":
                    if res[ol][:il] == "server":
                        server = res[ol][2 + il:-1]
                    elif res[ol][:il] == "port":
                        port = string.atoi(res[ol][2 + il:-1])					
    except:
        # Using defaults
        server = "127.0.0.1"
        port = 3418
    return server, port

# Some globals
Cv = threading.Condition()
if clientsocket.UseSSL == 1:
    crypto = clientsocket.crypto

# Set umask
os.umask(0066)

# Read settings
Server, Port = read_settings()

root = Tk()
gui = Rgui(root)
root.title("Remote nmap client")
root.iconname("Rnmap client")
root.update_idletasks()
root.minsize(root.winfo_reqwidth(), root.winfo_reqheight())
root.rowconfigure(15, weight=1)
root.columnconfigure(3, weight=1)

# Initialize
gui.check_scanoptions()
root.mainloop()
