# Copyright (C) 2003 Tuomo Makinen (tmakinen@pp.htv.fi)
# Licensed under the terms of the GNU General Public License
# clientsocket.py v. 0.0.1

try:
    from OpenSSL import SSL, crypto
except ImportError:
    UseSSL = -1
else:
    try:
        from OpenSSL import tsafe
    except ImportError:
        pass
    UseSSL = 1

from errno import EINTR
import socket

RECVBUFSIZE = 4096
CRLF = "\r\n"
CIPHER_LIST = "ALL:!ADH:!LOW:!EXP:!MD5:@STRENGTH"

class Client:
    """ Base class for SSLClient and RegularClient classes """
    def open_connection(self, server, port):
        try:
            self.sockobj.connect((server, port))
        except socket.error, e:
            return (-1, e[1])
        return (0,)

    def close_socket(self):
        self.sockobj.close()
        
class SSLClient(Client):
    def __init__(self, verify_callback_func, ca_certs, threaded=0):
        ctx = SSL.Context(SSL.TLSv1_METHOD)
        ctx.set_cipher_list(CIPHER_LIST)
        ctx.set_verify(SSL.VERIFY_PEER, verify_callback_func)
        try:
            ctx.load_verify_locations(ca_certs)
        except SSL.Error:
            # Can't load trusted CA certificates ...
            pass
        if threaded:
            # We want to be thread safe
            self.sockobj = tsafe.Connection(ctx, socket.socket(socket.AF_INET, socket.SOCK_STREAM))
        else:
            self.sockobj = SSL.Connection(ctx, socket.socket(socket.AF_INET, socket.SOCK_STREAM))
        
    def write_socket(self, message):
        message = message + CRLF
        totalsent = 0
        while totalsent < len(message):
            try:
                sent = self.sockobj.send(message[totalsent:])
            except (SSL.WantReadError, SSL.WantWriteError):
                continue
            except SSL.ZeroReturnError:
                return -1
            except SSL.Error:
                return -1
            totalsent = totalsent + sent

    def read_socket(self):
        while 1:
            chunk = ''
            try:
                chunk = self.sockobj.recv(RECVBUFSIZE)
            except (SSL.WantReadError, SSL.WantWriteError):
                continue
            except SSL.ZeroReturnError:
                return
            except SSL.Error:
                return -1
            return chunk

    def close_socket(self):
        self.sockobj.shutdown()
        self.sockobj.close()

class RegularClient(Client):
    def __init__(self):
        self.sockobj = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

    def write_socket(self, message):
        message = message + CRLF
        totalsent = 0
        while totalsent < len(message):
            try:
                sent = self.sockobj.send(message[totalsent:])
            except socket.error, e:
                if e[0] == EINTR:
                    continue
                else:
                    return -1
            totalsent = totalsent + sent

    def read_socket(self):
        while 1:
            chunk = ''
            try:
                chunk = self.sockobj.recv(RECVBUFSIZE)
            except socket.error, e:
                if e[0] == EINTR:
                    continue
                else:
                    return -1
            return chunk
