# Copyright (C) 2003 Tuomo Makinen (tmakinen@pp.htv.fi)
# Licensed under the terms of the GNU General Public License
# serversocket.py v. 0.0.1

try:
    from OpenSSL import SSL
except ImportError:
    UseSSL = -1
else:
    UseSSL = 1
    
from errno import EINTR
import socket

RECVBUFSIZE = 4096
CRLF = "\r\n"
BACKLOG = 5
CIPHER_LIST = "ALL:!ADH:!LOW:!EXP:!MD5:@STRENGTH"

class Server:
    """ Base class for SSLServer and RegularServer classes."""
    def activate_server(self, server, port):
        try:
            self.sockobj.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            self.sockobj.bind((server, port))
            self.sockobj.listen(BACKLOG)
        except socket.error, e:
            return (-1, e[1])
        return(0,)

    def accept_conn(self):
        return self.sockobj.accept()
        
    def close_socket(self):
        self.sockobj.close()

class SSLServer(Server):
    def __init__(self, pkey, cert):
        self.ctx = SSL.Context(SSL.SSLv23_METHOD)
        self.ctx.set_options(SSL.OP_NO_SSLv2)
        self.ctx.set_cipher_list(CIPHER_LIST)
        self.ctx.use_privatekey_file(pkey)
        self.ctx.use_certificate_file(cert)

    def do_socket(self):
        self.sockobj = SSL.Connection(self.ctx, socket.socket(socket.AF_INET, socket.SOCK_STREAM))
        
    def read_socket(self, conn):
        data = ''
        while 1:
            try:
                data = conn.recv(RECVBUFSIZE)
            except (SSL.WantReadError, SSL.WantWriteError):
                continue
            except SSL.ZeroReturnError:
                data = None
            except SSL.Error:
                data = None
            if not data:
                return
            elif data[-2:] == CRLF:
                return clean_message(data)

    def write_socket(self, conn, message):
        message = message + CRLF
        totalsent = 0
        while totalsent < len(message):
            try:
                sent = conn.send(message[totalsent:])
            except (SSL.WantReadError, SSL.WantWriteError):
                continue
            except SSL.ZeroReturnError:
                break
            except SSL.Error:
                break
            totalsent = totalsent + sent
            
class RegularServer(Server):
    def do_socket(self):
        self.sockobj = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

    def read_socket(self, conn):
        data = ''
        while 1:
            try:
                data = conn.recv(RECVBUFSIZE)
            except socket.error, e:
                data = None
            if not data:
                return
            elif data[-2:] == CRLF:
                return clean_message(data)

    def write_socket(self, conn, message):
        message = message + CRLF
        totalsent = 0
        while totalsent < len(message):
            try:
                sent = conn.send(message[totalsent:])
            except socket.error, e:
                break
            totalsent = totalsent + sent

def clean_message(message):
    if len(message) >= 2:
        if message[-2:] == CRLF:
            message = message[:-2]
        elif message[-1:] in CRLF:
            message = message[:-1]
    return message
